<?php

//* Start the engine

include_once( get_template_directory() . '/lib/init.php' );



//* Setup Theme

include_once( get_stylesheet_directory() . '/lib/theme-defaults.php' );



//* Set Localization (do not remove)

load_child_theme_textdomain( 'rapidmg', apply_filters( 'child_theme_textdomain', get_stylesheet_directory() . '/languages', 'rapidmg' ) );



//* Child theme (do not remove)

define( 'CHILD_THEME_NAME', __( 'RapidMG Theme', 'rapidmg' ) );

define( 'CHILD_THEME_URL', 'http://my.studiopress.com/themes/education/' );

define( 'CHILD_THEME_VERSION', '3.0.2' );



//* Add HTML5 markup structure

add_theme_support( 'html5', array( 'search-form', 'comment-form', 'comment-list', 'gallery', 'caption', ) );



//* Add viewport meta tag for mobile browsers

add_theme_support( 'genesis-responsive-viewport' );



// Register a custom image size for Singular Featured images

add_image_size( 'singular-featured-thumb', 1920, 600, true );

add_action( 'genesis_after_header', 'sk_display_featured_image' );

function sk_display_featured_image() {

if ( ! is_singular( array( 'post', 'page' ) ) ) {

return;

}

if ( ! has_post_thumbnail() ) {

return;

}



// Display featured image above content

echo '<div class="singular-featured-image">';

genesis_image( array( 'size' => 'singular-featured-thumb' ) );

echo '</div>';

}





//* Enqueue Scripts

add_action( 'wp_enqueue_scripts', 'rapidmg_load_scripts' );

function rapidmg_load_scripts() {



	wp_enqueue_script( 'rapidmg-responsive-menu', get_bloginfo( 'stylesheet_directory' ) . '/js/responsive-menu.js', array( 'jquery' ), '1.0.0' );

	

	wp_enqueue_style( 'dashicons' );



	wp_enqueue_style( 'google-fonts', '//fonts.googleapis.com/css?family=Roboto+Condensed:300,700', array(), CHILD_THEME_VERSION );

	

}



//* Add new image sizes

add_image_size( 'slider', 1600, 800, TRUE );

add_image_size( 'sidebar', 280, 150, TRUE );



//* Add support for custom header

add_theme_support( 'custom-header', array(

	'width'           => 311,

	'height'          => 106,

	'header-selector' => '.site-title a',

	'header-text'     => false,

) );



//* Add support for custom background

add_theme_support( 'custom-background' );



//* Add support for additional color style options

add_theme_support( 'genesis-style-selector', array(

	'rapidmg-blue'   => __( 'RapidMG Blue', 'rapidmg' ),

	'rapidmg-green'  => __( 'RapidMG Green', 'rapidmg' ),

	'rapidmg-red'    => __( 'RapidMG Red', 'rapidmg' ),

	'rapidmg-purple' => __( 'RapidMG Purple', 'rapidmg' ),

) );



//* Add support for 5-column footer widgets

add_theme_support( 'genesis-footer-widgets', 3 );



//* Add support for after entry widget

add_theme_support( 'genesis-after-entry-widget-area' );



//* Rename Primary and Secondary Menu

add_theme_support( 'genesis-menus' , array( 'primary' => __( 'Before Header Menu', 'rapidmg' ), 'secondary' => __( 'Footer Menu', 'rapidmg' ) ) );



//* Reposition the primary navigation menu

//remove_action( 'genesis_after_header', 'genesis_do_nav' );

//add_action( 'genesis_before_header', 'genesis_do_nav' );



//* Reposition the secondary navigation menu

remove_action( 'genesis_after_header', 'genesis_do_subnav' );

add_action( 'genesis_footer', 'genesis_do_subnav', 7 );



//* Reduce the secondary navigation menu to one level depth

add_filter( 'wp_nav_menu_args', 'rapidmg_secondary_menu_args' );

function rapidmg_secondary_menu_args( $args ){



	if( 'secondary' != $args['theme_location'] )

	return $args;



	$args['depth'] = 1;

	return $args;



}



//* Reposition the entry meta in the entry header

remove_action( 'genesis_entry_header', 'genesis_post_info', 12 );

add_action( 'genesis_entry_header', 'genesis_post_info', 5 );



//* Customize the entry meta in the entry header

add_filter( 'genesis_post_info', 'rapidmg_post_info_filter' );

function rapidmg_post_info_filter($post_info) {

	$post_info = '';

	return $post_info;

}



//* Customize the entry meta in the entry footer

add_filter( 'genesis_post_meta', 'rapidmg_post_meta_filter' );

function rapidmg_post_meta_filter($post_meta) {

	$post_meta = 'Article by [post_author_posts_link] [post_categories before=" &#47; "] [post_tags before=" &#47; "] [post_comments] [post_edit]';

	return $post_meta;

}



//* Relocate after post widget

remove_action( 'genesis_after_entry', 'genesis_after_entry_widget_area' );

add_action( 'genesis_after_entry', 'genesis_after_entry_widget_area', 5 );



//* Modify the size of the Gravatar in the author box

add_filter( 'genesis_author_box_gravatar_size', 'parallax_author_box_gravatar' );

function parallax_author_box_gravatar( $size ) {



	return 96;



}



//* Register widget areas

genesis_register_sidebar( array(

	'id'          => 'home-featured',

	'name'        => __( 'Home - Featured', 'rapidmg' ),

	'description' => __( 'This is the featured section of the Home page.', 'rapidmg' ),

) );

genesis_register_sidebar( array(

	'id'          => 'home-top',

	'name'        => __( 'Home - Top', 'rapidmg' ),

	'description' => __( 'This is the top section of the Home page.', 'rapidmg' ),

) );

genesis_register_sidebar( array(

	'id'          => 'home-middle',

	'name'        => __( 'Home - Middle', 'rapidmg' ),

	'description' => __( 'This is the middle section of the Home page.', 'rapidmg' ),

) );

genesis_register_sidebar( array(

	'id'          => 'home-bottom',

	'name'        => __( 'Home - Bottom', 'rapidmg' ),

	'description' => __( 'This is the bottom section of the Home page.', 'rapidmg' ),

) );



/** Customize Genesis Footer */

/** This snippet will help to modify the complete genesis footer area */

remove_action( 'genesis_footer', 'genesis_do_footer' );

add_action( 'genesis_footer', 'wpvkp_footer' );

function wpvkp_footer() {

    ?>

    <p>&copy; Copyright 2018 Rapid Manufacturing &middot; All Rights Reserved &middot; Website by <a href="http://olivermarketing.ca/" target="_blank">Oliver Marketing</a></p>

    <?php

}



remove_action( 'genesis_site_description', 'genesis_seo_site_description' );

remove_action( 'genesis_site_title', 'genesis_seo_site_title' );

add_action( 'genesis_site_title', 'wpc_seo_site_title' );



//add_action('genesis_custom_header_defaults', 'wpc_header_image');

//function wpc_header_image( $args ){

//    $args['width'] = 311;

//    $args['height'] = 106;

//    return $args;

//}



function wpc_seo_site_title(){

?>

    <a href="<?php bloginfo('siteurl'); ?>"><img src="<?php header_image(); ?>" /></a>

<?php

}



//add_action('init', 'wpc_init_function');

function wpc_init_function(){

    remove_action( 'genesis_after_header', 'genesis_do_nav' );

}



remove_action( 'genesis_after_header', 'genesis_do_nav' );

//add_action( 'genesis_header', 'genesis_do_nav' );



remove_action( 'genesis_header', 'genesis_do_header' );

add_action( 'genesis_header', 'wpc_do_header' );



function wpc_do_header() {



	global $wp_registered_sidebars;



	genesis_markup( array(

		'open'    => '<div %s>',

		'context' => 'title-area',

	) );



		do_action( 'genesis_site_title' );

		do_action( 'genesis_site_description' );



	genesis_markup( array(

		'close'    => '</div>',

		'context' => 'title-area',

	) );

    

    genesis_do_nav();

    

//	if ( ( isset( $wp_registered_sidebars['header-right'] ) && is_active_sidebar( 'header-right' ) ) || has_action( 'genesis_header_right' ) ) {

//

//    genesis_markup( array(

//        'open'    => '<div %s>' . genesis_sidebar_title( 'header-right' ),

//        'context' => 'header-widget-area',

//    ) );

//

//    do_action( 'genesis_header_right' );

//    add_filter( 'wp_nav_menu_args', 'genesis_header_menu_args' );

//    add_filter( 'wp_nav_menu', 'genesis_header_menu_wrap' );

//    dynamic_sidebar( 'header-right' );

//    remove_filter( 'wp_nav_menu_args', 'genesis_header_menu_args' );

//    remove_filter( 'wp_nav_menu', 'genesis_header_menu_wrap' );

//

//    genesis_markup( array(

//        'close'   => '</div>',

//        'context' => 'header-widget-area',

//    ) );

//

//	}



}

//* Do NOT include the opening php tag
 
add_filter( 'genesis_nav_items', 'sk_google_translator', 10, 2 );
add_filter( 'wp_nav_menu_items', 'sk_google_translator', 10, 2 );
 
function sk_google_translator($menu, $args) {
  $args = (array)$args;
	if ( 'primary' !== $args['theme_location'] )
		return $menu;
	ob_start();
		echo '<div id="google_translate_element"></div> ';
	$google_translator = ob_get_clean();
	return $menu . $google_translator;
}
